-- Anim

local AnimBlacklist = {"WORLD_HUMAN_MUSICIAN", "WORLD_HUMAN_CLIPBOARD"}
local AnimFemale = {
	["WORLD_HUMAN_BUM_WASH"] = {"amb@world_human_bum_wash@male@high@idle_a", "idle_a"},
	["WORLD_HUMAN_SIT_UPS"] = {"amb@world_human_sit_ups@male@idle_a", "idle_a"},
	["WORLD_HUMAN_PUSH_UPS"] = {"amb@world_human_push_ups@male@base", "base"},
	["WORLD_HUMAN_BUM_FREEWAY"] = {"amb@world_human_bum_freeway@male@base", "base"},
	["WORLD_HUMAN_CLIPBOARD"] = {"amb@world_human_clipboard@male@base", "base"},
	["WORLD_HUMAN_VEHICLE_MECHANIC"] = {"amb@world_human_vehicle_mechanic@male@base", "base"},
}

function RequestAndWaitDict(dictName)
	if dictName and DoesAnimDictExist(dictName) and not HasAnimDictLoaded(dictName) then
		RequestAnimDict(dictName)
		while not HasAnimDictLoaded(dictName) do Citizen.Wait(100) end
	end
end

function TableGetValue(tbl, value, k)
	if not tbl or not value or type(tbl) ~= "table" then return end
	for _,v in pairs(tbl) do
		if k and v[k] == value or v == value then return true, _ end
	end
end

function TaskAnim(animName, time, flag, ped, customPos)
	if type(animName) ~= "table" then animName = {animName} end
	ped, flag = ped or GetPlayerPed(-1), flag and tonumber(flag) or false

	if not animName or not animName[1] or string.len(animName[1]) < 1 then return end
    if IsEntityPlayingAnim(ped, animName[1], animName[2], 3) or IsPedActiveInScenario(ped) then ClearPedTasks(ped) 
        return end

	Citizen.CreateThread(function()
		TaskAnimForce(animName, flag, { ped = ped, time = time, pos = customPos })
	end)
end

function TaskAnimForce(animName, flag, args)
	flag, args = flag and tonumber(flag) or false, args or {}
	local ped, time, clearTasks, animPos, animRot, animTime = args.ped or GetPlayerPed(-1), args.time, args.clearTasks, args.pos, args.ang

	if IsPedInAnyVehicle(ped) and (not flag or flag < 40) then return end

	if not clearTasks then ClearPedTasks(ped) end

	if not animName[2] and AnimFemale[animName[1]] and GetEntityModel(ped) == -1667301416 then
		animName = AnimFemale[animName[1]]
	end

	if animName[2] and not HasAnimDictLoaded(animName[1]) then
		if not DoesAnimDictExist(animName[1]) then return end
		RequestAnimDict(animName[1])
		while not HasAnimDictLoaded(animName[1]) do
			Citizen.Wait(10)
		end
	end

	if not animName[2] then
		ClearAreaOfObjects(GetEntityCoords(ped), 1.0)
		TaskStartScenarioInPlace(ped, animName[1], -1, not TableGetValue(AnimBlacklist, animName[1]))
	else
        if not animPos then
            TaskPlayAnim(ped, animName[1], animName[2], 8.0, -8.0, -1, flag or 44, 1, 0, 0, 0, 0)
		else
			TaskPlayAnimAdvanced(ped, animName[1], animName[2], animPos.x, animPos.y, animPos.z, animRot.x, animRot.y, animRot.z, 8.0, -8.0, -1, 1, 1, 0, 0, 0)
		end
	end

	if time and type(time) == "number" then
		Citizen.Wait(time)
		ClearPedTasks(ped)
	end

	if not args.dict then RemoveAnimDict(animName[1]) end
end

-- GetNearby

function GetPlayers()
	local players = {}

	for _,player in ipairs(GetActivePlayers()) do
		local ped = GetPlayerPed(player)

		if DoesEntityExist(ped) then
			table.insert(players, player)
		end
	end

	return players
end

function GetNearbyPlayers(distance)
	local ped = GetPlayerPed(-1)
	local playerPos = GetEntityCoords(ped)
	local nearbyPlayers = {}

	for _,v in pairs(GetPlayers()) do
		local otherPed = GetPlayerPed(v)
		local otherPedPos = otherPed ~= ped and IsEntityVisible(otherPed) and GetEntityCoords(otherPed)

		if otherPedPos and GetDistanceBetweenCoords(otherPedPos, playerPos) <= (distance or max) then
			nearbyPlayers[#nearbyPlayers + 1] = v
		end
	end
	return nearbyPlayers
end

local cWait = false
local xWait = false
local max = 1.5

function GetClosestPlayer(istance, addVector)
	local ped, closestPlayer = GetPlayerPed(-1)
    local id = -1
    local cPed = nil
	local playerPos, playerForward = GetEntityCoords(ped), GetEntityForwardVector(ped)
	playerPos = playerPos + (addVector or playerForward * 0.5)

	for _,v in pairs(GetPlayers()) do
		local otherPed = GetPlayerPed(v)
		local otherPedPos = otherPed ~= ped and IsEntityVisible(otherPed) and GetEntityCoords(otherPed)

		if otherPedPos and GetDistanceBetweenCoords(otherPedPos, playerPos) <= (distance or max) and (not closestPlayer or GetDistanceBetweenCoords(otherPedPos, playerPos)) then
			closestPlayer = v
            cPed = GetPlayerPed(v)
		end
	end

	return closestPlayer, cPed
end

function GetNearbyPlayer(solo, other)
    
    if cWait then
        xWait = true
        while cWait do
            Citizen.Wait(5)
        end
    end

    xWait = false
    local cTimer = GetGameTimer() + 10000
    local oPlayer = GetNearbyPlayers(2)

    if solo then
        oPlayer[#oPlayer + 1] = PlayerId()
    end

    if #oPlayer == 0 then
		if SHARED.FRAMEWORK.ESX then
			TriggerEvent("esx:showNotification", SHARED.LOCALE.NO_PLAYER_CLOSEST)
		elseif SHARED.FRAMEWORK.QBCore then
			TriggerEvent('QBCore:Notify', SHARED.LOCALE.NO_PLAYER_CLOSEST)
		end 
        return false
    end

    if #oPlayer == 1 and other then
        return oPlayer[1]
    end

	if SHARED.FRAMEWORK.ESX then
		TriggerEvent("esx:showNotification", SHARED.LOCALE.ACTION_CLOSEST)
	elseif SHARED.FRAMEWORK.QBCore then
		TriggerEvent('QBCore:Notify', SHARED.LOCALE.ACTION_CLOSEST)
	end 

    Citizen.Wait(100)
    local cBase = 1
    cWait = true
    while GetGameTimer() <= cTimer and not xWait do
        Citizen.Wait(0)

        DisableControlAction(0, SHARED.CONTROL.CLOSEST_VALID, true)
        DisableControlAction(0, SHARED.CONTROL.CLOSEST_CHANGE, true)
        DisableControlAction(0, SHARED.CONTROL.CLOSEST_CANCEL, true)

        if IsDisabledControlJustPressed(0, SHARED.CONTROL.CLOSEST_VALID) then
            cWait = false
            return oPlayer[cBase]
        elseif IsDisabledControlJustPressed(0, SHARED.CONTROL.CLOSEST_CANCEL) then
			if SHARED.FRAMEWORK.ESX then
				TriggerEvent("esx:showNotification", SHARED.LOCALE.REFUSE_ACTION)
			elseif SHARED.FRAMEWORK.QBCore then
				TriggerEvent('QBCore:Notify', SHARED.LOCALE.REFUSE_ACTION)
			end 
            break
        elseif IsDisabledControlJustPressed(0, SHARED.CONTROL.CLOSEST_CHANGE) then
            cBase = (cBase == #oPlayer) and 1 or (cBase + 1)
        end
        local cPed = GetPlayerPed(oPlayer[cBase])
        local cCoords = GetEntityCoords(cPed)
        DrawMarker(0, cCoords.x, cCoords.y, cCoords.z + 1.0, 0.0, 0.0, 0.0, 180.0, 0.0, 0.0, 0.1, 0.1, 0.1, 0, 180, 10, 30, 1, 1, 0, 0, 0, 0, 0)
    end
    cWait = false
    return false
end